package org.ow2.jonas.cxf.transport.reliablejms.test.springmdp.impl;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import java.util.Map.Entry;

import org.ow2.jonas.cxf.transport.reliablejms.api.ContextInfo;

public class CheckerSingletonBean implements Checker {

    private Map<String, ContextInfo> calls = null;
    private Map<String, Object> answers = null;
    private Map<String, Throwable> errors = null;
    private Map<String, ContextInfo> delta = null;
    private Map<String, ContextInfo> invalidProtocol = null;
    
    public CheckerSingletonBean() {
        init();
    }   
    
    private void init() {
            this.calls = new HashMap<String, ContextInfo>();
            this.answers = new HashMap<String, Object>();
            this.errors = new HashMap<String, Throwable>();
            this.delta = new HashMap<String, ContextInfo>();
            this.invalidProtocol = new HashMap<String, ContextInfo>();
    }
    
    public void reset() {
        init();
    }
    
    public void addCall(ContextInfo contextInfo) {
        this.calls.put(contextInfo.getCorrelationID(), contextInfo);
        this.delta.put(contextInfo.getCorrelationID(), contextInfo);
    }


    public void handleResponse(ContextInfo contextInfo, Object response) {
        this.answers.put(contextInfo.getCorrelationID(), response);
        this.delta.remove(contextInfo.getCorrelationID());
    }

    public void handleFault(ContextInfo contextInfo, Throwable exception) {
        this.errors.put(contextInfo.getCorrelationID(), exception);
        this.delta.remove(contextInfo.getCorrelationID());
        
    }

    public boolean isKnownContext(ContextInfo contextInfo) {
        return calls.containsKey(contextInfo.getCorrelationID());
    }

    public void replyWithoutCall(ContextInfo contextInfo) {
        this.invalidProtocol.put(contextInfo.getCorrelationID(), contextInfo);
    }
    
    
    public int getCallsSize() {
        return calls.size();
    }
    
    public int getDeltaSize() {
        return delta.size();
    }
    
    public int getErrorsSize() {
        return errors.size();
    }
    
    public int getInvalidProtocolSize() {
        return invalidProtocol.size();
    }
    
    public Object getAnswer(String operationName) {
        String correlationID = getKeyForOperation(operationName);
        if (correlationID != null) {
            // found entry !
            if (answers.containsKey(correlationID)) {
                // return answer
                return answers.get(correlationID);
            } else {
                // no answer
                return Void.TYPE;
            }
        }
        throw new IllegalArgumentException("Operation named '" + operationName + "' was not found !");
    }

    protected String getKeyForOperation(String operationName) {
        // First, search operation in calls
        Set<Entry<String, ContextInfo>> callsEntries = calls.entrySet();
        if (callsEntries != null) {
            for (Entry<String, ContextInfo> callEntry : callsEntries) {
                ContextInfo contextInfo = callEntry.getValue();
                if (contextInfo.getWsdlOperation().getLocalPart().equals(operationName)) {
                    // found entry !
                    return callEntry.getKey();
                }
            }
        }
        return null;
        
    }
    
    
    public String getError(String operationName) {
        String correlationID = getKeyForOperation(operationName);
        if (correlationID != null) {
            Throwable t = errors.get(correlationID);
            if (t != null) {
                if (t.getMessage() != null) {
                    return t.getMessage();
                } else {
                    return "Message not found in the exception !!!!";
                }
            } else {
                return null;
            }
        }
        throw new IllegalArgumentException("Operation named '" + operationName + "' was not found !");
    }

	public String getAnswerChecksum(String operationName) {
		return null;
	}

	public String computeChecksum(Object o) {
		return null;
	}
     
    
}
