package org.ow2.jonas.cxf.transport.reliablejms.test.springmdp.impl;

import java.io.IOException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;

import org.ow2.jonas.cxf.transport.reliablejms.api.ContextInfo;

import javax.sql.DataSource;

public class PersistentCheckerSingletonBean implements Checker {

	private DataSource dataSource;
	
	private static final String T_CALLS = "CALLS";
	private static final String T_ANSWERS = "ANSWERS";
    private static final String T_ERRORS = "ERRORS";
	private static final String T_DELTA = "DELTA";
	private static final String T_INV_PROTOCOL = "INV_PROTOCOL";

	public void setDataSource(DataSource dataSource) {
		this.dataSource = dataSource;
	}

	private void init() {
		try {
			Connection conn = dataSource.getConnection();
			try {
				Statement stm = conn.createStatement();
				;
				try {
					stm.addBatch("DROP TABLE " + T_CALLS + " IF EXISTS;");
					stm.addBatch("DROP TABLE " + T_ANSWERS + " IF EXISTS;");
					stm.addBatch("DROP TABLE " + T_ERRORS + " IF EXISTS;");
					stm.addBatch("DROP TABLE " + T_DELTA + " IF EXISTS;");
					stm.addBatch("DROP TABLE " + T_INV_PROTOCOL + " IF EXISTS;");
					stm.addBatch("CREATE TABLE "
							+ T_CALLS
							+ " (ID varchar(256) primary key, OPERATION varchar(256) not null);");
					stm.addBatch("CREATE TABLE "
							+ T_ANSWERS
							+ " (ID varchar(256) primary key, CHECKSUM varchar(256) not null);");
					stm.addBatch("CREATE TABLE "
							+ T_ERRORS
							+ " (ID varchar(256) primary key, MSG varchar(256));");
					stm.addBatch("CREATE TABLE " + T_DELTA
							+ " (ID varchar(256) primary key);");
					stm.addBatch("CREATE TABLE " + T_INV_PROTOCOL
							+ " (ID varchar(256) primary key);");
					stm.executeBatch();
				} finally {
					stm.close();
				}
			} finally {
				conn.close();
			}
		} catch (SQLException e) {
			throw new RuntimeException(e);
		}
	}

	public void reset() {
		init();
	}

	public void addCall(ContextInfo contextInfo) {
		try {
			Connection conn = dataSource.getConnection();
			try {
				insertIntoCalls(conn, contextInfo);
				insertIntoDelta(conn, contextInfo);
			} finally {
				conn.close();
			}
		} catch (SQLException e) {
			throw new RuntimeException(e);
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}

	public void handleFault(ContextInfo contextInfo, Throwable exception) {
		try {
			Connection conn = dataSource.getConnection();
			try {
				insertIntoErrors(conn, contextInfo, exception);
				deleteFromDelta(conn, contextInfo.getCorrelationID());
			} finally {
				conn.close();
			}
		} catch (SQLException e) {
			throw new RuntimeException(e);
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}

	public void handleResponse(ContextInfo contextInfo, Object response) {
		try {
			Connection conn = dataSource.getConnection();
			try {
				insertIntoAnswers(conn, contextInfo, response);
				deleteFromDelta(conn, contextInfo.getCorrelationID());
			} finally {
				conn.close();
			}
		} catch (SQLException e) {
			throw new RuntimeException(e);
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}

	public void replyWithoutCall(ContextInfo contextInfo) {
		try {
			Connection conn = dataSource.getConnection();
			try {
				insertIntoInvProtocol(conn, contextInfo);
			} finally {
				conn.close();
			}
		} catch (SQLException e) {
			throw new RuntimeException(e);
		} catch (IOException e) {
			throw new RuntimeException(e);
		}

	}

	public boolean isKnownContext(ContextInfo contextInfo) {
		try {
			Connection conn = dataSource.getConnection();
			try {
				return selectFromCalls(conn, contextInfo.getCorrelationID()) != null;
			} finally {
				conn.close();
			}
		} catch (SQLException e) {
			throw new RuntimeException(e);
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}

	public int getCallsSize() {
		try {
			Connection conn = dataSource.getConnection();
			try {
				return nbRowInTable(conn, T_CALLS);
			} finally {
				conn.close();
			}
		} catch (SQLException e) {
			throw new RuntimeException(e);
		}
	}

	public int getDeltaSize() {
		try {
			Connection conn = dataSource.getConnection();
			try {
				return nbRowInTable(conn, T_DELTA);
			} finally {
				conn.close();
			}
		} catch (SQLException e) {
			throw new RuntimeException(e);
		}
	}

	public int getErrorsSize() {
		try {
			Connection conn = dataSource.getConnection();
			try {
				return nbRowInTable(conn, T_ERRORS);
			} finally {
				conn.close();
			}
		} catch (SQLException e) {
			throw new RuntimeException(e);
		}
	}

	public int getInvalidProtocolSize() {
		try {
			Connection conn = dataSource.getConnection();
			try {
				return nbRowInTable(conn, T_INV_PROTOCOL);
			} finally {
				conn.close();
			}
		} catch (SQLException e) {
			throw new RuntimeException(e);
		}
	}

	public String getAnswerChecksum(String operationName) {
		try {
			Connection conn = dataSource.getConnection();
			try {
				String correlationID = getKeyForOperation(conn, operationName);
				if (correlationID != null) {
					return selectFromAnswers(conn, correlationID);
				}
				throw new IllegalArgumentException("Operation named '"
						+ operationName + "' was not found !");
			} finally {
				conn.close();
			}
		} catch (SQLException e) {
			throw new RuntimeException(e);
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}

	public String getError(String operationName) {
		try {
			Connection conn = dataSource.getConnection();
			try {
				String correlationID = getKeyForOperation(conn, operationName);
				if (correlationID != null) {
					return selectFromErrors(conn, correlationID);
				}
				throw new IllegalArgumentException("Operation named '"
						+ operationName + "' was not found !");
			} finally {
				conn.close();
			}
		} catch (SQLException e) {
			throw new RuntimeException(e);
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}

	public String computeChecksum(Object o) {
		if (o == null) {
			return "null";
		} else {
			return o.getClass().getName() + Integer.toBinaryString(o.hashCode());
		}
	}
	
	protected String getKeyForOperation(Connection conn, String operationName)
			throws SQLException {
		for (String[] row : listCalls(conn)) {
			if (operationName.equals(row[1])) {
				return row[0];
			}
		}
		return null;
	}

	protected String selectFromCalls(Connection conn, String correlationID)
			throws SQLException, IOException {
		String query = "SELECT OPERATION FROM " + T_CALLS + " WHERE ID = ?";
		PreparedStatement stm = conn.prepareStatement(query);
		try {
			stm.setString(1, correlationID);
			ResultSet rs = stm.executeQuery();

			if (rs.next()) {
				return rs.getString(1);
			} else {
				return null;
			}
		} finally {
			stm.close();
		}
	}

	protected String selectFromAnswers(Connection conn, String correlationID)
			throws SQLException, IOException {
		String query = "SELECT CHECKSUM FROM " + T_ANSWERS + " WHERE ID = ?";
		PreparedStatement stm = conn.prepareStatement(query);
		try {
			stm.setString(1, correlationID);
			ResultSet rs = stm.executeQuery();

			if (rs.next()) {
				return rs.getString(1);
			} else {
				return computeChecksum(Void.TYPE);
			}
		} finally {
			stm.close();
		}
	}

	protected String selectFromErrors(Connection conn, String correlationID)
			throws SQLException, IOException {
		String query = "SELECT MSG FROM " + T_ERRORS + " WHERE ID = ?";
		PreparedStatement stm = conn.prepareStatement(query);
		try {
			stm.setString(1, correlationID);
			ResultSet rs = stm.executeQuery();

			if (rs.next()) {
				return rs.getString(1);
			} else {
				return null;
			}
		} finally {
			stm.close();
		}
	}

	protected void insertIntoCalls(Connection conn, ContextInfo contextInfo)
			throws SQLException, IOException {
		PreparedStatement stm = conn.prepareStatement("INSERT INTO " + T_CALLS
				+ " VALUES (?,?)");
		try {
			stm.setString(1, contextInfo.getCorrelationID());
			stm.setString(2, contextInfo.getWsdlOperation().getLocalPart());
			int retval = stm.executeUpdate();
			if (retval != 1) {
				throw new RuntimeException(
						String.format(
								"Fail to insert data into %s (correlationID=%s, retval=%d)",
								T_CALLS, contextInfo.getCorrelationID(), retval));
			}
		} finally {
			stm.close();
		}
	}

	protected void insertIntoAnswers(Connection conn, ContextInfo contextInfo,
			Object response) throws SQLException, IOException {
		PreparedStatement stm = conn.prepareStatement("INSERT INTO "
				+ T_ANSWERS + " VALUES (?,?)");
		try {
			stm.setString(1, contextInfo.getCorrelationID());
			stm.setString(2, computeChecksum(response));
			int retval = stm.executeUpdate();
			if (retval != 1) {
				throw new RuntimeException(
						String.format(
								"Fail to insert data into %s (correlationID=%s, retval=%d)",
								T_CALLS, contextInfo.getCorrelationID(), retval));

			}
		} finally {
			stm.close();
		}
	}

	protected void insertIntoErrors(Connection conn, ContextInfo contextInfo,
			Throwable throwable) throws SQLException, IOException {
		String errMsg = (throwable != null) ? ((throwable.getMessage() != null) ? throwable
				.getMessage() : "Message not found in the exception !!!!")
				: null;

		PreparedStatement stm = conn.prepareStatement("INSERT INTO " + T_ERRORS
				+ " VALUES (?,?)");
		try {
			stm.setString(1, contextInfo.getCorrelationID());
			stm.setString(2, errMsg);
			int retval = stm.executeUpdate();
			if (retval != 1) {
				throw new RuntimeException(
						String.format(
								"Fail to insert data into %s (correlationID=%s, retval=%d)",
								T_ERRORS, contextInfo.getCorrelationID(),
								retval));
			}
		} finally {
			stm.close();
		}
	}

	protected void insertIntoDelta(Connection conn, ContextInfo contextInfo)
			throws SQLException, IOException {
		PreparedStatement stm = conn.prepareStatement("INSERT INTO " + T_DELTA
				+ " VALUES (?)");
		try {
			stm.setString(1, contextInfo.getCorrelationID());
			int retval = stm.executeUpdate();
			if (retval != 1) {
				throw new RuntimeException(
						String.format(
								"Fail to insert data into %s (correlationID=%s, retval=%d)",
								T_DELTA, contextInfo.getCorrelationID(), retval));
			}
		} finally {
			stm.close();
		}
	}

	protected void insertIntoInvProtocol(Connection conn,
			ContextInfo contextInfo) throws SQLException, IOException {
		PreparedStatement stm = conn.prepareStatement("INSERT INTO "
				+ T_INV_PROTOCOL + " VALUES (?)");
		try {
			stm.setString(1, contextInfo.getCorrelationID());
			int retval = stm.executeUpdate();
			if (retval != 1) {
				throw new RuntimeException(
						String.format(
								"Fail to insert data into %s (correlationID=%s, retval=%d)",
								T_INV_PROTOCOL, contextInfo.getCorrelationID(),
								retval));
			}
		} finally {
			stm.close();
		}
	}

	protected int deleteFromDelta(Connection conn, String correlationID)
			throws SQLException {
		return deleteFromTable(conn, correlationID, T_DELTA);
	}

	protected int deleteFromTable(Connection conn, String id, String tableName)
			throws SQLException {
		String query = "DELETE FROM " + tableName + " WHERE ID = '" + id + "'";
		Statement stm = conn.createStatement();
		try {
			return stm.executeUpdate(query);
		} finally {
			stm.close();
		}
	}

	protected int nbRowInTable(Connection conn, String tableName)
			throws SQLException {
		String query = "SELECT COUNT(ID) FROM " + tableName;
		Statement stm = conn.createStatement();
		try {
			ResultSet rs = stm.executeQuery(query);

			if (rs.next()) {
				return rs.getInt(1);
			} else {
				throw new RuntimeException("SQL query '" + query
						+ "' returned no row");
			}
		} finally {
			stm.close();
		}
	}

	protected List<String[]> listCalls(Connection conn) throws SQLException {
		List<String[]> result = new ArrayList<String[]>();

		String query = "SELECT * FROM " + T_CALLS;
		Statement stm = conn.createStatement();
		try {
			ResultSet rs = stm.executeQuery(query);

			while (rs.next()) {
				String[] row = new String[2];
				row[0] = rs.getString(1);
				row[1] = rs.getString(2);
				result.add(row);
			}
		} finally {
			stm.close();
		}

		return result;
	}

}
