package org.ow2.jonas.cxf.transport.reliablejms.test.springmdp.impl;

import java.net.URL;

import org.apache.cxf.Bus;
import org.apache.cxf.bus.spring.BusApplicationContext;
import org.apache.cxf.transport.Destination;
import org.ow2.jonas.cxf.transport.reliablejms.IReliableJMSTransportSingletonBean;
import org.ow2.jonas.cxf.transport.reliablejms.test.springmdp.ComplexObject;
import org.ow2.jonas.cxf.transport.reliablejms.test.springmdp.WSTest;
import org.ow2.jonas.cxf.transport.reliablejms.ReliableJMSTransportFactory;
import org.springframework.context.ApplicationContext;

public class RemoteTesterBean implements RemoteTester, IReliableJMSTransportSingletonBean {

    public RemoteTesterBean() {

    }

    public static final String SPRING_RESOURCE = "/cxf-test.xml";

    private ReliableJMSTransportFactory reliableJMSTransportFactory = null;

    private WSTest service = null;

    private boolean initialized = false;

    private Checker checker;

    private int calls = 0;

    public void setChecker(Checker c) {
    	checker = c;
    }
    
    public void init() {
        if (!initialized) {
            URL url = getClass().getResource(SPRING_RESOURCE);
            if (url == null) {
                throw new IllegalStateException("The resource '" + SPRING_RESOURCE
                        + "' has not been found by using classloader of the class '" + getClass() + "'.");
            }

            // init endpoint
            ApplicationContext context = new BusApplicationContext(url, true);

            // Get the WS
            service = (WSTest) ((ServiceStorer) context.getBean("WebService")).getService();

            // Gets the transport factory that has been initialized
            this.reliableJMSTransportFactory = (ReliableJMSTransportFactory) context.getBean(ReliableJMSTransportFactory.class.getName());
            this.initialized = true;
			checker = (Checker) context.getBean("checker");
        }
        
        // reset data
        checker.reset();
        this.calls = 0;

    }

    public void callWS() throws Exception {

        ComplexObject complexObject = new ComplexObject();
        complexObject.setName("Florent");
        complexObject.setId(2503);
        complexObject.setAddress("Grenoble");

        service.anotherBasicOperation();
        calls++;

        service.basicString("Hello me");
        calls++;
        
        service.basicInteger(31337);
        calls++;

        service.basicBoolean(true);
        calls++;

        service.basicComplexObject(complexObject);
        calls++;

        service.basicException("throws");
        calls++;

        service.basicOperation();
        calls++;

        service.dummySimpleTypeInt(100);
        calls++;

        service.dummySimpleTypeBoolean(true);
        calls++;

        service.dummySimpleTypeByte((byte) 65);
        calls++;

        service.dummySimpleTypeShort((short) 25);
        calls++;

        service.dummySimpleTypeFloat(25.03f);
        calls++;

        service.dummySimpleTypeLong(250300l);
        calls++;

        service.dummySimpleTypeDouble(250300d);
        calls++;

        service.simpleString("hello me");
        calls++;

        service.simpleInteger(Integer.valueOf(2503));
        calls++;

        service.simpleBoolean(Boolean.TRUE);
        calls++;

        service.simpleComplexObject(complexObject);
        calls++;

        service.computeComplexObject("Florent", "Grenoble", 2503);
        calls++;

        service.uncheckedException();
        calls++;

        service.uncheckedExceptionParam("Server Exception");
        calls++;
    }

    public void callWSThenThrowRuntime() throws Exception {
    	callWS();
    	throw new RuntimeException("This should rollback all calls");
    }
    
    public int getCalls() {
        return calls;
    }

    public int getNotifiedCalls() {
        return checker.getCallsSize();
    }

    public void checkExpectedErrors() {
        int val = checker.getErrorsSize();
        int expected = 2;
        if (val != expected) {
            throw new IllegalStateException("Errors size is not expected. Expecting '" + expected + "' and got '" + val + "'");
        }

        // check error message
        checkError("basicException", "exception requested : throws");
        checkError("anotherBasicOperation", "exception requested");

    }

    public void checkExpectedValues() {
        checkAnswer("basicString", Void.TYPE);
        checkAnswer("basicInteger", Void.TYPE);
        checkAnswer("basicComplexObject", Void.TYPE);
        checkAnswer("basicException", Void.TYPE);
        checkAnswer("basicOperation", Void.TYPE);

        checkAnswer("dummySimpleTypeInt", 100);
        checkAnswer("dummySimpleTypeBoolean", true);
        checkAnswer("dummySimpleTypeByte", (byte) 65);
        checkAnswer("dummySimpleTypeShort", (short) 25);
        checkAnswer("dummySimpleTypeFloat", 25.03f);
        checkAnswer("dummySimpleTypeLong", 250300l);
        checkAnswer("dummySimpleTypeDouble", 250300d);
        checkAnswer("simpleString", "hello me");
        checkAnswer("simpleInteger", 2503);
        checkAnswer("simpleBoolean", true);
        checkAnswer("basicString", Void.TYPE);

        ComplexObject complexObject = new ComplexObject();
        complexObject.setName("Florent");
        complexObject.setId(2503);
        complexObject.setAddress("Grenoble");

        checkAnswer("simpleComplexObject", complexObject);
        checkAnswer("computeComplexObject", complexObject);

        // No answer for these methods and no error should be obtained
        checkAnswer("uncheckedException", Void.TYPE);
        checkAnswer("uncheckedExceptionParam", Void.TYPE);
        checkError("uncheckedExceptionParam", null);

    }

    protected void checkAnswer(String operationName, Object expected) {
    	String expectedChecksum = checker.computeChecksum(expected);
        String answerChecksum = checker.getAnswerChecksum(operationName);
        if (!expectedChecksum.equals(answerChecksum)) {
            throw new IllegalStateException("Expecting '" + expectedChecksum + "' for operation named '" + operationName + "', got : '" + answerChecksum + "'.");
        }
    }

    protected void checkError(String operationName, String expected) {
        String answer = checker.getError(operationName);
        if (expected == null && answer == null) {
            return;
        }
        if (!expected.equals(answer)) {
            throw new IllegalStateException("Expecting error '" + expected + "' for operation named '" + operationName + "', got : '" + answer + "'.");
        }
    }

    public Destination getDestination(String queueName) {
        return reliableJMSTransportFactory.getExistingDestination(queueName);
    }

    public Bus getBus() {
        return reliableJMSTransportFactory.getBus();
    }

}
